@@ -29,9 +29,9 @@
 #include <QAction>
 #include "discover_debug.h"
 #include <QDesktopServices>
-#include <QtQml/QQmlEngine>
-#include <QtQml/QQmlContext>
-#include <QtQml/QQmlApplicationEngine>
+#include <QQmlEngine>
+#include <QQmlContext>
+#include <QQmlApplicationEngine>
 #include <QtQuick/QQuickItem>
 #include <qqml.h>
 #include <QPointer>
@@ -40,6 +40,9 @@
 #include <QTimer>
 #include <QSessionManager>
 #include <QClipboard>
+#include <QDBusConnection>
+#include <QDBusInterface>
+#include <QDBusPendingCall>
 
 // KDE includes
 #include <KAboutApplicationDialog>
@@ -52,7 +55,7 @@
 #include <KSharedConfig>
 #include <KConfigGroup>
 #include <KStandardAction>
-#include <KIO/AccessManager>
+#include <kcoreaddons_version.h>
 // #include <KSwitchLanguageDialog>
 
 // DiscoverCommon includes
@@ -69,6 +72,18 @@
 #include <utils.h>
 #include <QMimeDatabase>
 
+#ifdef WITH_FEEDBACK
+#include "plasmauserfeedback.h"
+#endif
+#include "discoversettings.h"
+
+class CachedNetworkAccessManagerFactory : public QQmlNetworkAccessManagerFactory
+{
+    virtual QNetworkAccessManager * create(QObject *parent) override {
+        return new CachedNetworkAccessManager(QStringLiteral("images"), parent);
+    }
+};
+
 class OurSortFilterProxyModel : public QSortFilterProxyModel, public QQmlParserStatus
 {
     Q_OBJECT
@@ -81,18 +96,7 @@ public:
     }
 };
 
-class KIOAccessManagerFactory : public QQmlNetworkAccessManagerFactory
-{
-public:
-    KIOAccessManagerFactory() = default;
-    ~KIOAccessManagerFactory() = default;
-    QNetworkAccessManager *create(QObject *parent) override
-    {
-        return new KIO::AccessManager(parent);
-    }
-};
-
-DiscoverObject::DiscoverObject(CompactMode mode)
+DiscoverObject::DiscoverObject(CompactMode mode, const QVariantMap &initialProperties)
     : QObject()
     , m_engine(new QQmlApplicationEngine)
     , m_mode(mode)
@@ -103,17 +107,40 @@ DiscoverObject::DiscoverObject(CompactMode mode)
     auto factory = m_engine->networkAccessManagerFactory();
     m_engine->setNetworkAccessManagerFactory(nullptr);
     delete factory;
-    m_engine->setNetworkAccessManagerFactory(new KIOAccessManagerFactory());
+    m_engine->setNetworkAccessManagerFactory(m_networkAccessManagerFactory.data());
 
     qmlRegisterType<UnityLauncher>("org.kde.discover.app", 1, 0, "UnityLauncher");
     qmlRegisterType<PaginateModel>("org.kde.discover.app", 1, 0, "PaginateModel");
     qmlRegisterType<KConcatenateRowsProxyModel>("org.kde.discover.app", 1, 0, "KConcatenateRowsProxyModel");
     qmlRegisterType<FeaturedModel>("org.kde.discover.app", 1, 0, "FeaturedModel");
     qmlRegisterType<OurSortFilterProxyModel>("org.kde.discover.app", 1, 0, "QSortFilterProxyModel");
-
+#ifdef WITH_FEEDBACK
+    qmlRegisterSingletonType<PlasmaUserFeedback>("org.kde.discover.app", 1, 0, "UserFeedbackSettings", [](QQmlEngine*, QJSEngine*) -> QObject* { return new PlasmaUserFeedback(KSharedConfig::openConfig(QStringLiteral("PlasmaUserFeedback"), KConfig::NoGlobals)); });
+#endif
+    qmlRegisterSingletonType<DiscoverSettings>("org.kde.discover.app", 1, 0, "DiscoverSettings", [](QQmlEngine*, QJSEngine*) -> QObject* {
+            auto r = new DiscoverSettings;
+            connect(r, &DiscoverSettings::installedPageSortingChanged, r, &DiscoverSettings::save);
+            connect(r, &DiscoverSettings::appsListPageSortingChanged, r, &DiscoverSettings::save);
+            return r;
+        });
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
     qmlRegisterType<QQuickView>();
     qmlRegisterType<QActionGroup>();
     qmlRegisterType<QAction>();
+
+    qmlRegisterType<KAboutData>();
+    qmlRegisterType<KAboutLicense>();
+    qmlRegisterType<KAboutPerson>();
+#else
+    qmlRegisterAnonymousType<QQuickView>("org.kde.discover.app", 1);
+    qmlRegisterAnonymousType<QActionGroup>("org.kde.discover.app", 1);
+    qmlRegisterAnonymousType<QAction>("org.kde.discover.app", 1);
+
+    qmlRegisterAnonymousType<KAboutData>("org.kde.discover.app", 1);
+    qmlRegisterAnonymousType<KAboutLicense>("org.kde.discover.app", 1);
+    qmlRegisterAnonymousType<KAboutPerson>("org.kde.discover.app", 1);
+#endif
+
     qmlRegisterUncreatableType<DiscoverObject>("org.kde.discover.app", 1, 0, "DiscoverMainWindow", QStringLiteral("don't do that"));
     setupActions();
 
@@ -123,10 +150,11 @@ DiscoverObject::DiscoverObject(CompactMode mode)
     plugin->initializeEngine(m_engine, uri);
     plugin->registerTypes(uri);
 
-    //Here we set up a cache for the screenshots
-    delete m_engine->networkAccessManagerFactory();
-    m_engine->setNetworkAccessManagerFactory(m_networkAccessManagerFactory.data());
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+    m_engine->setInitialProperties(initialProperties);
+#endif
     m_engine->rootContext()->setContextProperty(QStringLiteral("app"), this);
+    m_engine->rootContext()->setContextProperty(QStringLiteral("discoverAboutData"), QVariant::fromValue(KAboutData::applicationData()));
 
     connect(m_engine, &QQmlApplicationEngine::objectCreated, this, &DiscoverObject::integrateObject);
     m_engine->load(QUrl(QStringLiteral("qrc:/qml/DiscoverWindow.qml")));
@@ -136,6 +164,23 @@ DiscoverObject::DiscoverObject(CompactMode mode)
         for(auto o: objs)
             delete o;
     });
+    auto action = new OneTimeAction(
+        [this]() {
+            bool found = DiscoverBackendsFactory::hasRequestedBackends();
+            const auto backends = ResourcesModel::global()->backends();
+            for (auto b : backends)
+                found |= b->hasApplications();
+
+            if (!found)
+                Q_EMIT openErrorPage(i18n("No application back-ends found, please report to your distribution."));
+        }
+        , this);
+
+    if (ResourcesModel::global()->backends().isEmpty()) {
+        connect(ResourcesModel::global(), &ResourcesModel::allInitialized, action, &OneTimeAction::trigger);
+    } else {
+        action->trigger();
+    }
 }
 
 DiscoverObject::~DiscoverObject()
@@ -189,23 +234,34 @@ void DiscoverObject::openMimeType(const QString& mime)
     emit listMimeInternal(mime);
 }
 
+void DiscoverObject::showLoadingPage()
+{
+    QObject* obj = rootObject();
+    if (!obj) {
+        qCWarning(DISCOVER_LOG) << "could not get the main object";
+        return;
+    }
+
+    obj->setProperty("currentTopLevel", QStringLiteral("qrc:/qml/LoadingPage.qml"));
+}
+
 void DiscoverObject::openCategory(const QString& category)
 {
-    setRootObjectProperty("defaultStartup", false);
+    showLoadingPage();
     auto action = new OneTimeAction(
         [this, category]() {
             Category* cat = CategoryModel::global()->findCategoryByName(category);
             if (cat) {
                 emit listCategoryInternal(cat);
             } else {
+                openMode(QStringLiteral("Browsing"));
                 showPassiveNotification(i18n("Could not find category '%1'", category));
-                setRootObjectProperty("defaultStartup", false);
             }
         }
         , this);
 
-    if (ResourcesModel::global()->backends().isEmpty()) {
-        connect(ResourcesModel::global(), &ResourcesModel::backendsChanged, action, &OneTimeAction::trigger);
+    if (CategoryModel::global()->rootCategories().isEmpty()) {
+        connect(CategoryModel::global(), &CategoryModel::rootCategoriesChanged, action, &OneTimeAction::trigger);
     } else {
         action->trigger();
     }
@@ -218,25 +274,28 @@ void DiscoverObject::openLocalPackage(const QUrl& localfile)
         qCWarning(DISCOVER_LOG) << "Trying to open unexisting file" << localfile;
         return;
     }
-    setRootObjectProperty("defaultStartup", false);
+    showLoadingPage();
     auto action = new OneTimeAction(
         [this, localfile]() {
-            auto res = ResourcesModel::global()->resourceForFile(localfile);
-            qCDebug(DISCOVER_LOG) << "all initialized..." << res;
-            if (res) {
-                emit openApplicationInternal(res);
-            } else {
-                QMimeDatabase db;
-                auto mime = db.mimeTypeForUrl(localfile);
-                auto fIsFlatpakBackend = [](AbstractResourcesBackend* backend) { return backend->metaObject()->className() == QByteArray("FlatpakBackend"); };
-                if (mime.name().startsWith(QLatin1String("application/vnd.flatpak")) && !kContains(ResourcesModel::global()->backends(), fIsFlatpakBackend)) {
-                    openApplication(QUrl(QLatin1String("appstream://org.kde.discover.flatpak")));
-                    showPassiveNotification(i18n("Cannot interact with flatpak resources without the flatpak backend %1. Please install it first.", localfile.toDisplayString()));
+            AbstractResourcesBackend::Filters f;
+            f.resourceUrl = localfile;
+            auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
+            connect(stream, &StoredResultsStream::finishedResources, this, [this, localfile](const QVector<AbstractResource*> &res) {
+                if (res.count() == 1) {
+                    emit openApplicationInternal(res.first());
                 } else {
-                    setRootObjectProperty("defaultStartup", true);
-                    showPassiveNotification(i18n("Couldn't open %1", localfile.toDisplayString()));
+                    QMimeDatabase db;
+                    auto mime = db.mimeTypeForUrl(localfile);
+                    auto fIsFlatpakBackend = [](AbstractResourcesBackend* backend) { return backend->metaObject()->className() == QByteArray("FlatpakBackend"); };
+                    if (mime.name().startsWith(QLatin1String("application/vnd.flatpak")) && !kContains(ResourcesModel::global()->backends(), fIsFlatpakBackend)) {
+                        openApplication(QUrl(QStringLiteral("appstream://org.kde.discover.flatpak")));
+                        showPassiveNotification(i18n("Cannot interact with flatpak resources without the flatpak backend %1. Please install it first.", localfile.toDisplayString()));
+                    } else {
+                        openMode(QStringLiteral("Browsing"));
+                        showPassiveNotification(i18n("Couldn't open %1", localfile.toDisplayString()));
+                    }
                 }
-            }
+            });
         }
         , this);
 
@@ -250,18 +309,16 @@ void DiscoverObject::openLocalPackage(const QUrl& localfile)
 void DiscoverObject::openApplication(const QUrl& url)
 {
     Q_ASSERT(!url.isEmpty());
-    setRootObjectProperty("defaultStartup", false);
+    showLoadingPage();
     auto action = new OneTimeAction(
         [this, url]() {
             AbstractResourcesBackend::Filters f;
             f.resourceUrl = url;
             auto stream = new StoredResultsStream({ResourcesModel::global()->search(f)});
-            connect(stream, &StoredResultsStream::finished, this, [this, url, stream]() {
-                const auto res = stream->resources();
+            connect(stream, &StoredResultsStream::finishedResources, this, [this, url](const QVector<AbstractResource*> &res) {
                 if (res.count() >= 1) {
                     emit openApplicationInternal(res.first());
                 } else {
-                    setRootObjectProperty("defaultStartup", true);
                     Q_EMIT openErrorPage(i18n("Couldn't open %1", url.toDisplayString()));
                 }
             });
@@ -425,7 +482,7 @@ public:
         if (m_warnings.isEmpty())
             qCDebug(DISCOVER_LOG) << "cool no warnings!";
         else
-            qCDebug(DISCOVER_LOG) << "test finished succesfully despite" << m_warnings;
+            qCDebug(DISCOVER_LOG) << "test finished successfully despite" << m_warnings;
         qGuiApp->exit(m_warnings.count());
     }
 
@@ -444,17 +501,6 @@ QWindow* DiscoverObject::rootObject() const
     return qobject_cast<QWindow*>(m_engine->rootObjects().at(0));
 }
 
-void DiscoverObject::setRootObjectProperty(const char* name, const QVariant& value)
-{
-    auto ro = rootObject();
-    if (!ro) {
-        qCWarning(DISCOVER_LOG) << "please check your installation";
-        return;
-    }
-
-    rootObject()->setProperty(name, value);
-}
-
 void DiscoverObject::showPassiveNotification(const QString& msg)
 {
     QTimer::singleShot(100, this, [this, msg](){
@@ -462,10 +508,21 @@ void DiscoverObject::showPassiveNotification(const QString& msg)
     });
 }
 
-void DiscoverObject::copyTextToClipboard(const QString& text)
+void DiscoverObject::reboot()
 {
-    qGuiApp->clipboard()->setText(text);
+    QDBusInterface interface(QStringLiteral("org.kde.ksmserver"), QStringLiteral("/KSMServer"), QStringLiteral("org.kde.KSMServerInterface"), QDBusConnection::sessionBus());
+    interface.asyncCall(QStringLiteral("logout"), 0, 1, 2); // Options: do not ask again | reboot | force
 }
 
+QRect DiscoverObject::initialGeometry() const
+{
+    KConfigGroup window(KSharedConfig::openConfig(), "Window");
+    return window.readEntry("geometry", QRect());
+}
+
+QString DiscoverObject::describeSources() const
+{
+    return rootObject()->property("describeSources").toString();
+}
 
 #include "DiscoverObject.moc"
