@@ -24,7 +24,6 @@
 #include <resources/AbstractResourcesBackend.h>
 #include <QVariantList>
 
-class QAction;
 class DummyReviewsBackend;
 class StandardBackendUpdater;
 class DummyResource;
@@ -39,7 +38,7 @@ public:
     AbstractBackendUpdater* backendUpdater() const override;
     AbstractReviewsBackend* reviewsBackend() const override;
     ResultsStream* search(const AbstractResourcesBackend::Filters & search) override;
-    ResultsStream * findResourceByPackageName(const QUrl& search) override;
+    ResultsStream * findResourceByPackageName(const QUrl& search);
     QHash<QString, DummyResource*> resources() const { return m_resources; }
     bool isValid() const override { return true; } // No external file dependencies that could cause runtime errors
 
@@ -47,7 +46,6 @@ public:
     Transaction* installApplication(AbstractResource* app, const AddonList& addons) override;
     Transaction* removeApplication(AbstractResource* app) override;
     bool isFetching() const override { return m_fetching; }
-    AbstractResource * resourceForFile(const QUrl & ) override;
     void checkForUpdates() override;
     QString displayName() const override;
     bool hasApplications() const override;
