@@ -27,8 +27,8 @@
 #include <QDirIterator>
 
 // Attica includes
-#include <attica/content.h>
-#include <attica/providermanager.h>
+#include <Attica/Content>
+#include <Attica/ProviderManager>
 
 // KDE includes
 #include <knewstuffcore_version.h>
@@ -65,11 +65,21 @@ class KNSBackendFactory : public AbstractResourcesBackendFactory {
         QVector<AbstractResourcesBackend*> newInstance(QObject* parent, const QString &/*name*/) const override
         {
             QVector<AbstractResourcesBackend*> ret;
-            for (const QString &path: QStandardPaths::standardLocations(QStandardPaths::GenericConfigLocation)) {
+#if KNEWSTUFFCORE_VERSION_MAJOR==5 && KNEWSTUFFCORE_VERSION_MINOR>=57
+            QStringList locations = KNSCore::Engine::configSearchLocations();
+#else
+            QStringList locations = QStandardPaths::standardLocations(QStandardPaths::GenericConfigLocation);
+#endif
+            QSet<QString> files;
+            for (const QString &path: locations) {
                 QDirIterator dirIt(path, {QStringLiteral("*.knsrc")}, QDir::Files);
                 for(; dirIt.hasNext(); ) {
                     dirIt.next();
 
+                    if (files.contains(dirIt.fileName()))
+                        continue;
+                    files << dirIt.fileName();
+
                     auto bk = new KNSBackend(parent, QStringLiteral("plasma"), dirIt.filePath());
                     if (bk->isValid())
                         ret += bk;
@@ -96,7 +106,7 @@ KNSBackend::KNSBackend(QObject* parent, const QString& iconName, const QString &
     setName(fileName);
     setObjectName(knsrc);
 
-    const KConfig conf(m_name);
+    const KConfig conf(m_name, KConfig::SimpleConfig);
     if (!conf.hasGroup("KNewStuff3")) {
         markInvalid(QStringLiteral("Config group not found! Check your KNS3 installation."));
         return;
@@ -110,31 +120,6 @@ KNSBackend::KNSBackend(QObject* parent, const QString& iconName, const QString &
 
     setFetching(true);
 
-    m_engine = new KNSCore::Engine(this);
-    m_engine->init(m_name);
-    m_engine->setPageSize(100);
-    // Setting setFetching to false when we get an error ensures we don't end up in an eternally-fetching state
-    connect(m_engine, &KNSCore::Engine::signalError, this, [this](const QString &_error) {
-        QString error = _error;
-        bool invalidFile = false;
-        if(error == QLatin1Literal("All categories are missing")) {
-            markInvalid(error);
-            error = i18n("Invalid %1 backend, contact your distributor.", m_displayName);
-            invalidFile = true;
-        }
-        m_responsePending = false;
-        Q_EMIT searchFinished();
-        Q_EMIT availableForQueries();
-        this->setFetching(false);
-        qWarning() << "kns error" << objectName() << error;
-        if (!invalidFile)
-            Q_EMIT passiveMessage(i18n("%1: %2", name(), error));
-    });
-    connect(m_engine, &KNSCore::Engine::signalEntriesLoaded, this, &KNSBackend::receivedEntries, Qt::QueuedConnection);
-    connect(m_engine, &KNSCore::Engine::signalEntryChanged, this, &KNSBackend::statusChanged, Qt::QueuedConnection);
-    connect(m_engine, &KNSCore::Engine::signalEntryDetailsLoaded, this, &KNSBackend::statusChanged);
-    connect(m_engine, &KNSCore::Engine::signalProvidersLoaded, this, &KNSBackend::fetchInstalled);
-
     // This ensures we have something to track when checking after the initialization timeout
     connect(this, &KNSBackend::initialized, this, [this](){ m_initialized = true; });
     // If we have not initialized in 60 seconds, consider this KNS backend invalid
@@ -154,23 +139,90 @@ KNSBackend::KNSBackend(QObject* parent, const QString& iconName, const QString &
         m_displayName = fileName.mid(0, fileName.indexOf(QLatin1Char('.')));
         m_displayName[0] = m_displayName[0].toUpper();
     }
+    m_hasApplications = group.readEntry<bool>("X-Discover-HasApplications", false);
 
-    static const QSet<QString> knsrcPlasma = {
-        QStringLiteral("aurorae.knsrc"), QStringLiteral("icons.knsrc"), QStringLiteral("kfontinst.knsrc"), QStringLiteral("lookandfeel.knsrc"), QStringLiteral("plasma-themes.knsrc"), QStringLiteral("plasmoids.knsrc"),
-        QStringLiteral("wallpaper.knsrc"), QStringLiteral("xcursor.knsrc"),
+    const QStringList cats = group.readEntry<QStringList>("Categories", QStringList{});
+    QVector<Category*> categories;
+    if (cats.count() > 1) {
+        m_categories += cats;
+        for(const auto &cat: cats) {
+            if (m_hasApplications)
+                categories << new Category(cat, QStringLiteral("applications-other"), { {CategoryFilter, cat } }, backendName, {}, {}, true);
+            else
+                categories << new Category(cat, QStringLiteral("plasma"), { {CategoryFilter, cat } }, backendName, {}, {}, true);
+        }
+    }
 
-        QStringLiteral("cgcgtk3.knsrc"), QStringLiteral("cgcicon.knsrc"), QStringLiteral("cgctheme.knsrc"), //GTK integration
-        QStringLiteral("kwinswitcher.knsrc"), QStringLiteral("kwineffect.knsrc"), QStringLiteral("kwinscripts.knsrc"), //KWin
-        QStringLiteral("comic.knsrc"), QStringLiteral("colorschemes.knsrc"),
-        QStringLiteral("emoticons.knsrc"), QStringLiteral("plymouth.knsrc"),
-        QStringLiteral("sddmtheme.knsrc")
-    };
-    auto actualCategory = new Category(m_displayName, QStringLiteral("plasma"), filters, backendName, {}, QUrl(), true);
+    QVector<Category*> topCategories{categories};
+    for (const auto &cat: categories) {
+        const QString catName = cat->name().append(QLatin1Char('/'));
+        for (const auto& potentialSubCat: categories) {
+            if(potentialSubCat->name().startsWith(catName)) {
+                cat->addSubcategory(potentialSubCat);
+                topCategories.removeOne(potentialSubCat);
+            }
+        }
+    }
 
-    const auto topLevelName = knsrcPlasma.contains(fileName)? i18n("Plasma Addons") : i18n("Application Addons");
-    const QUrl decoration(knsrcPlasma.contains(fileName)? QStringLiteral("https://c2.staticflickr.com/4/3148/3042248532_20bd2e38f4_b.jpg") : QStringLiteral("https://c2.staticflickr.com/8/7067/6847903539_d9324dcd19_o.jpg"));
-    auto addonsCategory = new Category(topLevelName, QStringLiteral("plasma"), filters, backendName, {actualCategory}, decoration, true);
-    m_rootCategories = { addonsCategory };
+    m_engine = new KNSCore::Engine(this);
+    connect(m_engine, &KNSCore::Engine::signalErrorCode, this, &KNSBackend::signalErrorCode);
+    connect(m_engine, &KNSCore::Engine::signalEntriesLoaded, this, &KNSBackend::receivedEntries, Qt::QueuedConnection);
+    connect(m_engine, &KNSCore::Engine::signalEntryChanged, this, &KNSBackend::statusChanged, Qt::QueuedConnection);
+    connect(m_engine, &KNSCore::Engine::signalEntryDetailsLoaded, this, &KNSBackend::detailsLoaded);
+    connect(m_engine, &KNSCore::Engine::signalProvidersLoaded, this, &KNSBackend::fetchInstalled);
+    connect(m_engine, &KNSCore::Engine::signalCategoriesMetadataLoded, this, [categories](const QList< KNSCore::Provider::CategoryMetadata>& categoryMetadatas){
+        for (const KNSCore::Provider::CategoryMetadata& category : categoryMetadatas) {
+            for (Category* cat : categories) {
+                if (cat->orFilters().count() > 0 && cat->orFilters().first().second == category.name) {
+                    cat->setName(category.displayName);
+                    break;
+                }
+            }
+        }
+    });
+    m_engine->setPageSize(100);
+    m_engine->init(m_name);
+
+    if(m_hasApplications) {
+        auto actualCategory = new Category(m_displayName, QStringLiteral("applications-other"), filters, backendName, topCategories, QUrl(), false);
+        auto applicationCategory = new Category(i18n("Applications"), QStringLiteral("applications-internet"), filters, backendName, { actualCategory }, QUrl(), false);
+        applicationCategory->setAndFilter({ {CategoryFilter, QLatin1String("Application")} });
+        m_categories.append(applicationCategory->name());
+        m_rootCategories = { applicationCategory };
+        // Make sure we filter out any apps which won't run on the current system architecture
+        QStringList tagFilter = m_engine->tagFilter();
+        if(QSysInfo::currentCpuArchitecture() == QLatin1String("arm")) {
+            tagFilter << QLatin1String("application##architecture==armhf");
+        } else if(QSysInfo::currentCpuArchitecture() == QLatin1String("arm64")) {
+            tagFilter << QLatin1String("application##architecture==arm64");
+        } else if(QSysInfo::currentCpuArchitecture() == QLatin1String("i386")) {
+            tagFilter << QLatin1String("application##architecture==x86");
+        } else if(QSysInfo::currentCpuArchitecture() == QLatin1String("ia64")) {
+            tagFilter << QLatin1String("application##architecture==x86-64");
+        } else if(QSysInfo::currentCpuArchitecture() == QLatin1String("x86_64")) {
+            tagFilter << QLatin1String("application##architecture==x86");
+            tagFilter << QLatin1String("application##architecture==x86-64");
+        }
+        m_engine->setTagFilter(tagFilter);
+    } else {
+        static const QSet<QString> knsrcPlasma = {
+            QStringLiteral("aurorae.knsrc"), QStringLiteral("icons.knsrc"), QStringLiteral("kfontinst.knsrc"), QStringLiteral("lookandfeel.knsrc"), QStringLiteral("plasma-themes.knsrc"), QStringLiteral("plasmoids.knsrc"),
+            QStringLiteral("wallpaper.knsrc"), QStringLiteral("xcursor.knsrc"),
+
+            QStringLiteral("cgcgtk3.knsrc"), QStringLiteral("cgcicon.knsrc"), QStringLiteral("cgctheme.knsrc"), //GTK integration
+            QStringLiteral("kwinswitcher.knsrc"), QStringLiteral("kwineffect.knsrc"), QStringLiteral("kwinscripts.knsrc"), //KWin
+            QStringLiteral("comic.knsrc"), QStringLiteral("colorschemes.knsrc"),
+            QStringLiteral("emoticons.knsrc"), QStringLiteral("plymouth.knsrc"),
+            QStringLiteral("sddmtheme.knsrc"), QStringLiteral("wallpaperplugin.knsrc"),
+            QStringLiteral("ksplash.knsrc"), QStringLiteral("window-decorations.knsrc")
+        };
+        const auto iconName = knsrcPlasma.contains(fileName)? QStringLiteral("plasma") : QStringLiteral("applications-other");
+        auto actualCategory = new Category(m_displayName, iconName, filters, backendName, categories, QUrl(), true);
+
+        const auto topLevelName = knsrcPlasma.contains(fileName)? i18n("Plasma Addons") : i18n("Application Addons");
+        auto addonsCategory = new Category(topLevelName, iconName, filters, backendName, {actualCategory}, QUrl(), true);
+        m_rootCategories = { addonsCategory };
+    }
 }
 
 KNSBackend::~KNSBackend()
@@ -180,6 +232,7 @@ KNSBackend::~KNSBackend()
 
 void KNSBackend::markInvalid(const QString &message)
 {
+    m_rootCategories.clear();
     qWarning() << "invalid kns backend!" << m_name << "because:" << message;
     m_isValid = false;
     setFetching(false);
@@ -224,7 +277,16 @@ KNSResource* KNSBackend::resourceForEntry(const KNSCore::EntryInternal& entry)
 {
     KNSResource* r = static_cast<KNSResource*>(m_resourcesByName.value(entry.uniqueId()));
     if (!r) {
-        r = new KNSResource(entry, m_categories, this);
+        QStringList categories{name(), m_rootCategories.first()->name()};
+        const auto cats = m_engine->categoriesMetadata();
+        const int catIndex = kIndexOf(cats, [&entry](const KNSCore::Provider::CategoryMetadata& cat){ return entry.category() == cat.id; });
+        if (catIndex > -1) {
+            categories << cats.at(catIndex).name;
+        }
+        if(m_hasApplications) {
+            categories << QLatin1String("Application");
+        }
+        r = new KNSResource(entry, categories, this);
         m_resourcesByName.insert(entry.uniqueId(), r);
     } else {
         r->setEntry(entry);
@@ -235,36 +297,99 @@ KNSResource* KNSBackend::resourceForEntry(const KNSCore::EntryInternal& entry)
 void KNSBackend::receivedEntries(const KNSCore::EntryInternal::List& entries)
 {
     m_responsePending = false;
+    const auto filtered = kFilter<KNSCore::EntryInternal::List>(entries, [](const KNSCore::EntryInternal& entry){ return entry.isValid(); });
+    const auto resources = kTransform<QVector<AbstractResource*>>(filtered, [this](const KNSCore::EntryInternal& entry){ return resourceForEntry(entry); });
 
-    const auto resources = kTransform<QVector<AbstractResource*>>(entries, [this](const KNSCore::EntryInternal& entry){ return resourceForEntry(entry); });
     if (!resources.isEmpty()) {
         Q_EMIT receivedResources(resources);
-    }
-
-    if(resources.isEmpty()) {
+    } else {
         Q_EMIT searchFinished();
         Q_EMIT availableForQueries();
         setFetching(false);
         return;
     }
 //     qDebug() << "received" << objectName() << this << m_resourcesByName.count();
-    if (!m_responsePending && !m_onePage) {
-        // We _have_ to set this first. If we do not, we may run into a situation where the
-        // data request will conclude immediately, causing m_responsePending to remain true
-        // for perpetuity as the slots will be called before the function returns.
-        m_responsePending = true;
-        m_engine->requestMoreData();
-    } else {
+    if (m_onePage) {
         Q_EMIT availableForQueries();
         setFetching(false);
     }
 }
 
+void KNSBackend::fetchMore()
+{
+    if (m_responsePending)
+        return;
+
+    // We _have_ to set this first. If we do not, we may run into a situation where the
+    // data request will conclude immediately, causing m_responsePending to remain true
+    // for perpetuity as the slots will be called before the function returns.
+    m_responsePending = true;
+    m_engine->requestMoreData();
+}
+
 void KNSBackend::statusChanged(const KNSCore::EntryInternal& entry)
 {
     resourceForEntry(entry);
 }
 
+void KNSBackend::signalErrorCode(const KNSCore::ErrorCode& errorCode, const QString& message, const QVariant& metadata)
+{
+    QString error = message;
+    qDebug() << "KNS error in" << m_displayName << ":" << errorCode << message << metadata;
+    bool invalidFile = false;
+    switch(errorCode) {
+        case KNSCore::ErrorCode::UnknownError:
+            // This is not supposed to be hit, of course, but any error coming to this point should be non-critical and safely ignored.
+            break;
+        case KNSCore::ErrorCode::NetworkError:
+            // If we have a network error, we need to tell the user about it. This is almost always fatal, so mark invalid and tell the user.
+            error = i18n("Network error in backend %1: %2", m_displayName, metadata.toInt());
+            markInvalid(error);
+            invalidFile = true;
+            break;
+        case KNSCore::ErrorCode::OcsError:
+            if(metadata.toInt() == 200) {
+                // Too many requests, try again in a couple of minutes - perhaps we can simply postpone it automatically, and give a message?
+                error = i18n("Too many requests sent to the server for backend %1. Please try again in a few minutes.", m_displayName);
+            } else {
+                // Unknown API error, usually something critical, mark as invalid and cry a lot
+                error = i18n("Invalid %1 backend, contact your distributor.", m_displayName);
+                markInvalid(error);
+                invalidFile = true;
+            }
+            break;
+        case KNSCore::ErrorCode::ConfigFileError:
+            error = i18n("Invalid %1 backend, contact your distributor.", m_displayName);
+            markInvalid(error);
+            invalidFile = true;
+            break;
+        case KNSCore::ErrorCode::ProviderError:
+            error = i18n("Invalid %1 backend, contact your distributor.", m_displayName);
+            markInvalid(error);
+            invalidFile = true;
+            break;
+        case KNSCore::ErrorCode::InstallationError:
+            // This error is handled already, by forwarding the KNS engine's installer error message.
+            break;
+        case KNSCore::ErrorCode::ImageError:
+            // Image fetching errors are not critical as such, but may lead to weird layout issues, might want handling...
+            error = i18n("Could not fetch screenshot for the entry %1 in backend %2", metadata.toList().at(0).toString(), m_displayName);
+            break;
+        default:
+            // Having handled all current error values, we should by all rights never arrive here, but for good order and future safety...
+            error = i18n("Unhandled error in %1 backend. Contact your distributor.", m_displayName);
+            break;
+    }
+    m_responsePending = false;
+    Q_EMIT searchFinished();
+    Q_EMIT availableForQueries();
+    // Setting setFetching to false when we get an error ensures we don't end up in an eternally-fetching state
+    this->setFetching(false);
+    qWarning() << "kns error" << objectName() << error;
+    if (!invalidFile)
+        Q_EMIT passiveMessage(i18n("%1: %2", name(), error));
+}
+
 class KNSTransaction : public Transaction
 {
 public:
@@ -290,7 +415,8 @@ public:
             };
         else
             actionFunction = [res, engine]() {
-                for(auto i : res->linkIds())
+                const auto links = res->linkIds();
+                for(auto i : links)
                     engine->install(res->entry(), i);
             };
         QTimer::singleShot(0, res, actionFunction);
@@ -383,8 +509,10 @@ ResultsStream* KNSBackend::search(const AbstractResourcesBackend::Filters& filte
         }
 
         return stream;
-    } else if ((!filter.category && !filter.search.isEmpty()) || (filter.category && filter.category->matchesCategoryName(m_categories.constFirst()))) {
-        auto r = new ResultsStream(QStringLiteral("KNS-search-")+name());
+    } else if ((!filter.category && !filter.search.isEmpty()) // Accept global searches
+              // If there /is/ a category, make sure we actually are one of those requested before searching
+           || (filter.category && kContains(m_categories, [&filter](const QString& cat) { return filter.category->matchesCategoryName(cat); }))) {
+        auto r = new ResultsStream(QLatin1String("KNS-search-")+name());
         searchStream(r, filter.search);
         return r;
     }
@@ -406,6 +534,7 @@ void KNSBackend::searchStream(ResultsStream* stream, const QString &searchText)
         m_onePage = false;
         m_responsePending = true;
 
+        connect(stream, &ResultsStream::fetchMore, this, &KNSBackend::fetchMore);
         connect(this, &KNSBackend::receivedResources, stream, &ResultsStream::resourcesFound);
         connect(this, &KNSBackend::searchFinished, stream, &ResultsStream::finish);
         connect(this, &KNSBackend::startingSearch, stream, &ResultsStream::finish);
@@ -433,12 +562,13 @@ ResultsStream * KNSBackend::findResourceByPackageName(const QUrl& search)
     const auto providerid = pathParts.at(0);
     const auto entryid = pathParts.at(1);
 
-    auto stream = new ResultsStream(QStringLiteral("KNS-byname-")+entryid);
+    auto stream = new ResultsStream(QLatin1String("KNS-byname-")+entryid);
     auto start = [this, entryid, stream, providerid]() {
         m_responsePending = true;
         m_engine->fetchEntryById(entryid);
         m_onePage = false;
-        connect(m_engine, &KNSCore::Engine::signalError, stream, &ResultsStream::finish);
+
+        connect(m_engine, &KNSCore::Engine::signalErrorCode, stream, &ResultsStream::finish);
         connect(m_engine, &KNSCore::Engine::signalEntryDetailsLoaded, stream, [this, stream, entryid, providerid](const KNSCore::EntryInternal &entry) {
             if (entry.uniqueId() == entryid && providerid == QUrl(entry.providerId()).host()) {
                 Q_EMIT stream->resourcesFound({resourceForEntry(entry)});
@@ -472,4 +602,10 @@ QString KNSBackend::displayName() const
     return QStringLiteral("KNewStuff");
 }
 
+void KNSBackend::detailsLoaded(const KNSCore::EntryInternal& entry)
+{
+    auto res = resourceForEntry(entry);
+    res->longDescriptionChanged();
+}
+
 #include "KNSBackend.moc"
