@@ -77,10 +77,10 @@ QString KNSResource::comment()
         ret = m_entry.summary();
         int newLine = ret.indexOf(QLatin1Char('\n'));
         if(newLine>0) {
-            ret=ret.left(newLine);
+            ret.truncate(newLine);
         }
-        ret = ret.replace(QRegularExpression(QStringLiteral("\\[/?[a-z]*\\]")), QString());
-        ret = ret.remove(QRegularExpression(QStringLiteral("<[^>]*>")));
+        ret.remove(QRegularExpression(QStringLiteral("\\[\\/?[a-z]*\\]")));
+        ret.remove(QRegularExpression(QStringLiteral("<[^>]*>")));
     }
     return ret;
 }
@@ -95,9 +95,14 @@ QString KNSResource::longDescription()
         else
             ret = ret.mid(newLine+1).trimmed();
     }
-    ret = ret.replace(QLatin1Char('\r'), QString());
-    ret = ret.replace(QStringLiteral("[li]"), QStringLiteral("\n* "));
-    ret = ret.replace(QRegularExpression(QStringLiteral("\\[/?[a-z]*\\]")), QString());
+    ret.remove(QLatin1Char('\r'));
+    ret.replace(QStringLiteral("[li]"), QStringLiteral("\n* "));
+    // Get rid of all BBCode markup we don't handle above
+    ret.remove(QRegularExpression(QStringLiteral("\\[\\/?[a-z]*\\]")));
+    // Find anything that looks like a link (but which also is not some html
+    // tag value or another already) and make it a link
+    static const QRegularExpression urlRegExp(QStringLiteral("(^|\\s)([-a-zA-Z0-9@:%_\\+.~#?&//=]{2,256}\\.[a-z]{2,4}\\b(\\/[-a-zA-Z0-9@:;%_\\+.~#?&//=]*)?)"), QRegularExpression::CaseInsensitiveOption);
+    ret.replace(urlRegExp, QStringLiteral("<a href=\"\\2\">\\2</a>"));
     return ret;
 }
 
@@ -136,9 +141,9 @@ KNSCore::EntryInternal KNSResource::entry() const
     return m_entry;
 }
 
-QString KNSResource::license()
+QJsonArray KNSResource::licenses()
 {
-    return m_entry.license();
+    return { QJsonObject{ {QStringLiteral("name"), m_entry.license()}, {QStringLiteral("url"), QString()} } };
 }
 
 int KNSResource::size()
@@ -229,6 +234,9 @@ void KNSResource::invokeApplication() const
 
 QString KNSResource::executeLabel() const
 {
+    if(knsBackend()->hasApplications()) {
+        return i18n("Launch");
+    }
     return i18n("Use");
 }
 
@@ -240,7 +248,8 @@ QDate KNSResource::releaseDate() const
 QVector<int> KNSResource::linkIds() const
 {
     QVector<int> ids;
-    for(const auto &e : m_entry.downloadLinkInformationList()) {
+    const auto linkInfo = m_entry.downloadLinkInformationList();
+    for(const auto &e : linkInfo) {
         if (e.isDownloadtypeLink)
             ids << e.id;
     }
@@ -261,8 +270,13 @@ Rating * KNSResource::ratingInstance()
         return new Rating(
             packageName(),
             noc,
-            { { QStringLiteral("star5"), rating } }
+            rating / 10
         );
     }
     return m_rating;
 }
+
+QString KNSResource::author() const
+{
+    return m_entry.author().name();
+}
