@@ -42,7 +42,7 @@ KNSBackendTest::KNSBackendTest(QObject* parent)
     , m_r(nullptr)
 {
     QStandardPaths::setTestModeEnabled(true);
-    ResourcesModel* model = new ResourcesModel(QLatin1String("kns-backend"), this);
+    ResourcesModel* model = new ResourcesModel(QStringLiteral("kns-backend"), this);
     Q_ASSERT(!model->backends().isEmpty());
     auto findTestBackend = [](AbstractResourcesBackend* backend) {
         return backend->name() == QLatin1String("discover_ktexteditor_codesnippets_core.knsrc");
@@ -63,7 +63,10 @@ QVector<AbstractResource*> KNSBackendTest::getResources(ResultsStream* stream, b
     Q_ASSERT(stream->objectName() != QLatin1String("KNS-void"));
     QSignalSpy spyResources(stream, &ResultsStream::destroyed);
     QVector<AbstractResource*> resources;
-    connect(stream, &ResultsStream::resourcesFound, this, [&resources](const QVector<AbstractResource*>& res) { resources += res; });
+    connect(stream, &ResultsStream::resourcesFound, this, [&resources, stream](const QVector<AbstractResource*>& res) {
+        resources += res;
+        Q_EMIT stream->fetchMore();
+    });
     Q_ASSERT(spyResources.wait(10000));
     Q_ASSERT(!resources.isEmpty() || canBeEmpty);
     return resources;
@@ -136,7 +139,7 @@ void KNSBackendTest::reviewsArrived(AbstractResource* r, const QVector<ReviewPtr
 void KNSBackendTest::testResourceByUrl()
 {
     AbstractResourcesBackend::Filters f;
-    f.resourceUrl = QUrl(QStringLiteral("kns://") + m_backend->name() + QStringLiteral("/api.kde-look.org/1136471"));
+    f.resourceUrl = QUrl(QLatin1String("kns://") + m_backend->name() + QLatin1String("/api.kde-look.org/1136471"));
     const QVector<AbstractResource*> resources = getResources(m_backend->search(f));
     const QVector<QUrl> res = kTransform<QVector<QUrl>>(resources, [](AbstractResource* res){ return res->url(); });
     QCOMPARE(res.count(), 1);
