@@ -28,6 +28,7 @@
 #include <KLocalizedString>
 #include <KToolInvocation>
 #include <QIcon>
+#include <QFile>
 #include <QProcess>
 #include <QStandardPaths>
 #include <QDebug>
@@ -43,19 +44,20 @@ AppPackageKitResource::AppPackageKitResource(const AppStream::Component& data, c
 
 QString AppPackageKitResource::name() const
 {
-    QString ret;
-    if (!m_appdata.extends().isEmpty()) {
-        auto components = backend()->componentsById(m_appdata.extends().constFirst());
+    if (m_name.isEmpty()) {
+        if (!m_appdata.extends().isEmpty()) {
+            const auto components = backend()->componentsById(m_appdata.extends().constFirst());
 
-        if (components.isEmpty())
-            qWarning() << "couldn't find" << m_appdata.extends() << "which is supposedly extended by" << m_appdata.id();
-        else
-            ret = components.constFirst().name() + QStringLiteral(" - ") + m_appdata.name();
+            if (components.isEmpty())
+                qWarning() << "couldn't find" << m_appdata.extends() << "which is supposedly extended by" << m_appdata.id();
+            else
+                m_name = components.constFirst().name() + QLatin1String(" - ") + m_appdata.name();
+        }
+
+        if (m_name.isEmpty())
+            m_name = m_appdata.name();
     }
-
-    if (ret.isEmpty())
-        ret = m_appdata.name();
-    return ret;
+    return m_name;
 }
 
 QString AppPackageKitResource::longDescription()
@@ -100,10 +102,9 @@ QVariant AppPackageKitResource::icon() const
     return componentIcon(m_appdata);
 }
 
-QString AppPackageKitResource::license()
+QJsonArray AppPackageKitResource::licenses()
 {
-    const auto license = m_appdata.projectLicense();
-    return license.isEmpty() ? PackageKitResource::license() : license;
+    return m_appdata.projectLicense().isEmpty() ? PackageKitResource::licenses() : AppStreamUtils::licenses(m_appdata);
 }
 
 QStringList AppPackageKitResource::mimetypes() const
@@ -111,10 +112,12 @@ QStringList AppPackageKitResource::mimetypes() const
     return m_appdata.provided(AppStream::Provided::KindMimetype).items();
 }
 
+static const QVector<AppStream::Component::Kind> s_addonKinds = {AppStream::Component::KindAddon, AppStream::Component::KindCodec};
+
 QStringList AppPackageKitResource::categories()
 {
     auto cats = m_appdata.categories();
-    if (m_appdata.kind() != AppStream::Component::KindAddon)
+    if (!s_addonKinds.contains(m_appdata.kind()))
         cats.append(QStringLiteral("Application"));
     return cats;
 }
@@ -153,11 +156,13 @@ QUrl AppPackageKitResource::donationURL()
     return m_appdata.url(AppStream::Component::UrlKindDonation);
 }
 
-bool AppPackageKitResource::isTechnical() const
+AbstractResource::Type AppPackageKitResource::type() const
 {
     static QString desktop = QString::fromUtf8(qgetenv("XDG_CURRENT_DESKTOP"));
     const auto desktops = m_appdata.compulsoryForDesktops();
-    return (!desktops.isEmpty() && !desktops.contains(desktop)) || m_appdata.kind() == AppStream::Component::KindAddon;
+    return s_addonKinds.contains(m_appdata.kind())               ? Addon
+           : (desktops.isEmpty() || !desktops.contains(desktop)) ? Application
+                                                                 : Technical;
 }
 
 void AppPackageKitResource::fetchScreenshots()
@@ -177,8 +182,8 @@ QStringList AppPackageKitResource::allPackageNames() const
 
 QList<PackageState> AppPackageKitResource::addonsInformation()
 {
-    return kTransform<QList<PackageState>>(
-        backend()->extendedBy(m_appdata.id()),
+    const auto res = kFilter<QVector<AppPackageKitResource*>>(backend()->extendedBy(m_appdata.id()), [this](AppPackageKitResource* r){ return r->allPackageNames() != allPackageNames(); });
+    return kTransform<QList<PackageState>>(res,
         [](AppPackageKitResource* r) { return PackageState(r->appstreamId(), r->name(), r->comment(), r->isInstalled()); }
     );
 }
@@ -193,19 +198,34 @@ QString AppPackageKitResource::changelog() const
     return AppStreamUtils::changelogToHtml(m_appdata);
 }
 
+
+bool AppPackageKitResource::canExecute() const
+{
+    static QSet<QString> cannotExecute = { QStringLiteral("org.kde.development") };
+    return !cannotExecute.contains(m_appdata.id());
+}
+
 void AppPackageKitResource::invokeApplication() const
 {
     auto trans = PackageKit::Daemon::getFiles({installedPackageId()});
-    connect(trans, &PackageKit::Transaction::files, this, [this](const QString &/*packageID*/, const QStringList &filenames) {
+    connect(trans, &PackageKit::Transaction::errorCode, backend(), &PackageKitBackend::transactionError);
+    connect(trans, &PackageKit::Transaction::files, this, [this](const QString &/*packageID*/, const QStringList &_filenames) {
+        //This workarounds bug in zypper's backend (suse) https://github.com/hughsie/PackageKit/issues/351
+        QStringList filenames = _filenames;
+        if (filenames.count() == 1 && !QFile::exists(filenames.constFirst())) {
+            filenames = filenames.constFirst().split(QLatin1Char(';'));
+        }
         const auto allServices = QStandardPaths::locateAll(QStandardPaths::ApplicationsLocation, m_appdata.id());
         if (!allServices.isEmpty()) {
             const auto packageServices = kFilter<QStringList>(allServices, [filenames](const QString &file) { return filenames.contains(file); });
             QProcess::startDetached(QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/discover/runservice"), {packageServices});
+            return;
         } else {
             const QStringList exes = m_appdata.provided(AppStream::Provided::KindBinary).items();
-            const auto packageExecutables = kFilter<QStringList>(allServices, [filenames](const QString &exe) { return filenames.contains(QLatin1Char('/') + exe); });
+            const auto packageExecutables = kFilter<QStringList>(exes, [filenames](const QString &exe) { return filenames.contains(QLatin1Char('/') + exe); });
             if (!packageExecutables.isEmpty()) {
-                QProcess::startDetached(exes.constFirst());
+                QProcess::startDetached(exes.constFirst(), QStringList());
+                return;
             } else {
                 const auto locations = QStandardPaths::standardLocations(QStandardPaths::ApplicationsLocation);
                 const auto desktopFiles = kFilter<QStringList>(filenames, [locations](const QString &exe) {
@@ -217,9 +237,10 @@ void AppPackageKitResource::invokeApplication() const
                 });
                 if (!desktopFiles.isEmpty()) {
                     QProcess::startDetached(QStringLiteral(CMAKE_INSTALL_FULL_LIBEXECDIR_KF5 "/discover/runservice"), { desktopFiles });
+                    return;
                 }
             }
-            qWarning() << "Could not find any executables" << exes << filenames;
+            Q_EMIT backend()->passiveMessage(i18n("Cannot launch %1", name()));
         }
     });
 }
@@ -234,6 +255,11 @@ QDate AppPackageKitResource::releaseDate() const
     return {};
 }
 
+QString AppPackageKitResource::author() const
+{
+    return m_appdata.developerName();
+}
+
 void AppPackageKitResource::fetchChangelog()
 {
     emit changelogFetched(changelog());
