@@ -32,6 +32,7 @@
 #include <resources/SourcesModel.h>
 #include <appstream/OdrsReviewsBackend.h>
 #include <appstream/AppStreamIntegration.h>
+#include <appstream/AppStreamUtils.h>
 
 #include <QProcess>
 #include <QStringList>
@@ -39,17 +40,23 @@
 #include <QTimer>
 #include <QStandardPaths>
 #include <QFile>
+#include <QAction>
+#include <QMimeDatabase>
+#include <QFileSystemWatcher>
+#include <QFutureWatcher>
+#include <QtConcurrentRun>
 
 #include <PackageKit/Transaction>
 #include <PackageKit/Daemon>
+#include <PackageKit/Offline>
 #include <PackageKit/Details>
 
 #include <KLocalizedString>
-#include <QAction>
-#include <QMimeDatabase>
+#include <KProtocolManager>
 
 #include "utils.h"
 #include "config-paths.h"
+#include "libdiscover_backend_debug.h"
 
 DISCOVER_BACKEND_PLUGIN(PackageKitBackend)
 
@@ -88,23 +95,56 @@ PackageKitBackend::PackageKitBackend(QObject* parent)
     m_delayedDetailsFetch.setInterval(100);
     connect(&m_delayedDetailsFetch, &QTimer::timeout, this, &PackageKitBackend::performDetailsFetch);
 
-    connect(PackageKit::Daemon::global(), &PackageKit::Daemon::updatesChanged, this, &PackageKitBackend::fetchUpdates);
+    connect(PackageKit::Daemon::global(), &PackageKit::Daemon::restartScheduled, m_updater, &PackageKitUpdater::enableNeedsReboot);
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::isRunningChanged, this, &PackageKitBackend::checkDaemonRunning);
-    connect(m_reviews.data(), &OdrsReviewsBackend::ratingsReady, this, &AbstractResourcesBackend::emitRatingsReady);
+    connect(m_reviews.data(), &OdrsReviewsBackend::ratingsReady, this, [this] {
+        m_reviews->emitRatingFetched(this, kTransform<QList<AbstractResource*>>(m_packages.packages.values(), [] (AbstractResource* r) { return r; }));
+    });
+
+    auto proxyWatch = new QFileSystemWatcher(this);
+    proxyWatch->addPath(QStandardPaths::writableLocation(QStandardPaths::ConfigLocation) + QLatin1String("/kioslaverc"));
+    connect(proxyWatch, &QFileSystemWatcher::fileChanged, this, [this](){
+        KProtocolManager::reparseConfiguration();
+        updateProxy();
+    });
 
     SourcesModel::global()->addSourcesBackend(new PackageKitSourcesBackend(this));
 
     reloadPackageList();
 
+    acquireFetching(true);
     setWhenAvailable(PackageKit::Daemon::getTimeSinceAction(PackageKit::Transaction::RoleRefreshCache), [this](uint timeSince) {
         if (timeSince > 3600)
             checkForUpdates();
         else
             fetchUpdates();
+        acquireFetching(false);
     }, this);
 }
 
-PackageKitBackend::~PackageKitBackend() = default;
+PackageKitBackend::~PackageKitBackend()
+{
+    m_threadPool.waitForDone(200);
+    m_threadPool.clear();
+}
+
+void PackageKitBackend::updateProxy()
+{
+
+    if (PackageKit::Daemon::isRunning()) {
+        static bool everHad = KProtocolManager::useProxy();
+        if (!everHad && !KProtocolManager::useProxy())
+            return;
+
+        everHad = KProtocolManager::useProxy();
+        PackageKit::Daemon::global()->setProxy(KProtocolManager::proxyFor(QStringLiteral("http")),
+                                               KProtocolManager::proxyFor(QStringLiteral("https")),
+                                               KProtocolManager::proxyFor(QStringLiteral("ftp")),
+                                               KProtocolManager::proxyFor(QStringLiteral("socks")),
+                                               {},
+                                               {});
+    }
+}
 
 bool PackageKitBackend::isFetching() const
 {
@@ -120,10 +160,50 @@ void PackageKitBackend::acquireFetching(bool f)
 
     if ((!f && m_isFetching==0) || (f && m_isFetching==1)) {
         emit fetchingChanged();
+        if (m_isFetching==0)
+            emit available();
     }
     Q_ASSERT(m_isFetching>=0);
 }
 
+struct DelayedAppStreamLoad {
+    QVector<AppStream::Component> components;
+    QHash<QString, AppStream::Component> missingComponents;
+    bool correct = true;
+};
+
+static DelayedAppStreamLoad loadAppStream(AppStream::Pool* appdata)
+{
+    DelayedAppStreamLoad ret;
+
+    QString error;
+    ret.correct = appdata->load(&error);
+    if (!ret.correct) {
+        qWarning() << "Could not open the AppStream metadata pool" << error;
+    }
+
+    const auto components = appdata->components();
+    ret.components.reserve(components.size());
+    foreach(const AppStream::Component& component, components) {
+        if (component.kind() == AppStream::Component::KindFirmware)
+            continue;
+
+        const auto pkgNames = component.packageNames();
+        if (pkgNames.isEmpty()) {
+            const auto entries = component.launchable(AppStream::Launchable::KindDesktopId).entries();
+            if (component.kind() == AppStream::Component::KindDesktopApp && !entries.isEmpty()) {
+                const QString file = PackageKitBackend::locateService(entries.first());
+                if (!file.isEmpty()) {
+                    ret.missingComponents[file] = component;
+                }
+            }
+        } else {
+            ret.components << component;
+        }
+    }
+    return ret;
+}
+
 void PackageKitBackend::reloadPackageList()
 {
     acquireFetching(true);
@@ -131,68 +211,62 @@ void PackageKitBackend::reloadPackageList()
         disconnect(m_refresher.data(), &PackageKit::Transaction::finished, this, &PackageKitBackend::reloadPackageList);
     }
 
-    QString error;
     m_appdata.reset(new AppStream::Pool);
-    const bool b = m_appdata->load(&error);
-    if (!b && m_packages.packages.isEmpty()) {
-        qWarning() << "Could not open the AppStream metadata pool" << error;
 
-        QTimer::singleShot(0, this, [this]() {
-            Q_EMIT passiveMessage(i18n("Please make sure that Appstream is properly set up on your system"));
-        });
-    }
+    auto fw = new QFutureWatcher<DelayedAppStreamLoad>(this);
+    connect(fw, &QFutureWatcher<DelayedAppStreamLoad>::finished, this, [this, fw]() {
+        const auto data = fw->result();
+        fw->deleteLater();
 
-    const auto components = m_appdata->components();
-    QStringList neededPackages;
-    neededPackages.reserve(components.size());
-    foreach(const AppStream::Component& component, components) {
-        if (component.kind() == AppStream::Component::KindFirmware)
-            continue;
-
-        const auto pkgNames = component.packageNames();
-        if (pkgNames.isEmpty()) {
-            auto launchable = component.launchable(AppStream::Launchable::KindDesktopId);
-            if (component.kind() == AppStream::Component::KindDesktopApp && !launchable.entries().isEmpty()) {
-                const QString file = locateService(launchable.entries().constFirst());
-                if (!file.isEmpty()) {
-                    acquireFetching(true);
-                    auto trans = PackageKit::Daemon::searchFiles(file);
-                    connect(trans, &PackageKit::Transaction::package, this, [trans](PackageKit::Transaction::Info info, const QString &packageID){
-                        if (info == PackageKit::Transaction::InfoInstalled)
-                            trans->setProperty("installedPackage", packageID);
-                    });
-                    connect(trans, &PackageKit::Transaction::finished, this, [this, trans, component](PackageKit::Transaction::Exit status) {
-                        const auto pkgidVal = trans->property("installedPackage");
-                        if (status == PackageKit::Transaction::ExitSuccess && !pkgidVal.isNull()) {
-                            const auto pkgid = pkgidVal.toString();
-                            auto res = addComponent(component, {PackageKit::Daemon::packageName(pkgid)});
-                            res->clearPackageIds();
-                            res->addPackageId(PackageKit::Transaction::InfoInstalled, pkgid, true);
-                        }
-                        acquireFetching(false);
-                    });
-                    continue;
+        if (!data.correct && m_packages.packages.isEmpty()) {
+            QTimer::singleShot(0, this, [this]() {
+                Q_EMIT passiveMessage(i18n("Please make sure that Appstream is properly set up on your system"));
+            });
+        }
+        QStringList neededPackages;
+        neededPackages.reserve(data.components.size());
+        for (const auto &component: data.components) {
+            const auto pkgNames = component.packageNames();
+            addComponent(component, pkgNames);
+            neededPackages << pkgNames;
+        }
+        for (auto it = data.missingComponents.constBegin(), itEnd = data.missingComponents.constEnd(); it != itEnd; ++it) {
+            acquireFetching(true);
+            const auto file = it.key();
+            const auto component = it.value();
+            auto trans = PackageKit::Daemon::searchFiles(file);
+            connect(trans, &PackageKit::Transaction::package, this, [trans](PackageKit::Transaction::Info info, const QString &packageID){
+                if (info == PackageKit::Transaction::InfoInstalled)
+                    trans->setProperty("installedPackage", packageID);
+            });
+            connect(trans, &PackageKit::Transaction::finished, this, [this, trans, component](PackageKit::Transaction::Exit status) {
+                const auto pkgidVal = trans->property("installedPackage");
+                if (status == PackageKit::Transaction::ExitSuccess && !pkgidVal.isNull()) {
+                    const auto pkgid = pkgidVal.toString();
+                    auto res = addComponent(component, {PackageKit::Daemon::packageName(pkgid)});
+                    res->clearPackageIds();
+                    res->addPackageId(PackageKit::Transaction::InfoInstalled, pkgid, true);
                 }
+                acquireFetching(false);
+            });
+        }
+
+        if (!neededPackages.isEmpty()) {
+            neededPackages.removeDuplicates();
+            resolvePackages(neededPackages);
+        } else {
+            qCDebug(LIBDISCOVER_BACKEND_LOG) << "empty appstream db";
+            if (PackageKit::Daemon::backendName() == QLatin1String("aptcc") || PackageKit::Daemon::backendName().isEmpty()) {
+                checkForUpdates();
             }
-
-            qDebug() << "no packages for" << component.id();
-            continue;
         }
-        neededPackages += pkgNames;
-
-        addComponent(component, pkgNames);
-    }
-
-    acquireFetching(false);
-    if (!neededPackages.isEmpty()) {
-        neededPackages.removeDuplicates();
-        resolvePackages(neededPackages);
-    } else {
-        qDebug() << "empty appstream db";
-        if (PackageKit::Daemon::backendName() == QLatin1String("aptcc") || PackageKit::Daemon::backendName().isEmpty()) {
-            checkForUpdates();
+        if (!m_appstreamInitialized) {
+            m_appstreamInitialized = true;
+            Q_EMIT loadedAppStream();
         }
-    }
+        acquireFetching(false);
+    });
+    fw->setFuture(QtConcurrent::run(&m_threadPool, &loadAppStream, m_appdata.get()));
 }
 
 AppPackageKitResource* PackageKitBackend::addComponent(const AppStream::Component& component, const QStringList& pkgNames)
@@ -218,14 +292,6 @@ AppPackageKitResource* PackageKitBackend::addComponent(const AppStream::Componen
     return res;
 }
 
-void PackageKitBackend::clearPackages(const QStringList& packageNames)
-{
-    const auto resources = resourcesByPackageNames<QVector<AbstractResource*>>(packageNames);
-    for(auto res: resources) {
-        qobject_cast<PackageKitResource*>(res)->clearPackageIds();
-    }
-}
-
 void PackageKitBackend::resolvePackages(const QStringList &packageNames)
 {
     PackageKit::Transaction * tArch = PackageKit::Daemon::resolve(packageNames, PackageKit::Transaction::FilterArch);
@@ -245,14 +311,17 @@ void PackageKitBackend::fetchUpdates()
     if (m_updater->isProgressing())
         return;
 
-    PackageKit::Transaction * tUpdates = PackageKit::Daemon::getUpdates();
-    connect(tUpdates, &PackageKit::Transaction::finished, this, &PackageKitBackend::getUpdatesFinished);
-    connect(tUpdates, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackageToUpdate);
-    connect(tUpdates, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
+    m_getUpdatesTransaction = PackageKit::Daemon::getUpdates();
+    connect(m_getUpdatesTransaction, &PackageKit::Transaction::finished, this, &PackageKitBackend::getUpdatesFinished);
+    connect(m_getUpdatesTransaction, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackageToUpdate);
+    connect(m_getUpdatesTransaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
+    connect(m_getUpdatesTransaction, &PackageKit::Transaction::percentageChanged, this, &PackageKitBackend::fetchingUpdatesProgressChanged);
     m_updatesPackageId.clear();
     m_hasSecurityUpdates = false;
 
     m_updater->setProgressing(true);
+    
+    fetchingUpdatesProgressChanged();
 }
 
 void PackageKitBackend::addPackageArch(PackageKit::Transaction::Info info, const QString& packageId, const QString& summary)
@@ -267,6 +336,12 @@ void PackageKitBackend::addPackageNotArch(PackageKit::Transaction::Info info, co
 
 void PackageKitBackend::addPackage(PackageKit::Transaction::Info info, const QString &packageId, const QString &summary, bool arch)
 {
+    if(PackageKit::Daemon::packageArch(packageId) == QLatin1String("source")) {
+        // We do not add source packages, they make little sense here. If source is needed,
+        // we are going to have to consider that in some other way, some other time
+        // If we do not ignore them here, e.g. openSuse entirely fails at installing applications
+        return;
+    }
     const QString packageName = PackageKit::Daemon::packageName(packageId);
     QSet<AbstractResource*> r = resourcesByPackageName(packageName);
     if (r.isEmpty()) {
@@ -305,7 +380,8 @@ void PackageKitBackend::includePackagesToAdd()
             auto res = m_packages.packages.take(pkg);
             if (res) {
                 if (AppPackageKitResource* ares = qobject_cast<AppPackageKitResource*>(res)) {
-                    for(const auto &ext: res->extends())
+                    const auto extends = res->extends();
+                    for(const auto &ext: extends)
                         m_packages.extendedBy[ext].removeAll(ares);
                 }
 
@@ -359,13 +435,19 @@ T PackageKitBackend::resourcesByPackageNames(const QStringList &pkgnames) const
 
 void PackageKitBackend::checkForUpdates()
 {
+    if (PackageKit::Daemon::global()->offline()->updateTriggered()) {
+        qCDebug(LIBDISCOVER_BACKEND_LOG) << "Won't be checking for updates again, the system needs a reboot to apply the fetched offline updates.";
+        return;
+    }
+
     if (!m_refresher) {
         acquireFetching(true);
         m_refresher = PackageKit::Daemon::refreshCache(false);
+
         connect(m_refresher.data(), &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
         connect(m_refresher.data(), &PackageKit::Transaction::finished, this, [this]() {
             m_refresher = nullptr;
-            reloadPackageList();
+            fetchUpdates();
             acquireFetching(false);
         });
     } else {
@@ -375,6 +457,7 @@ void PackageKitBackend::checkForUpdates()
 
 QList<AppStream::Component> PackageKitBackend::componentsById(const QString& id) const
 {
+    Q_ASSERT(m_appstreamInitialized);
     return m_appdata->componentsById(id);
 }
 
@@ -383,47 +466,60 @@ ResultsStream* PackageKitBackend::search(const AbstractResourcesBackend::Filters
     if (!filter.resourceUrl.isEmpty()) {
         return findResourceByPackageName(filter.resourceUrl);
     } else if (!filter.extends.isEmpty()) {
-        const auto ext = kTransform<QVector<AbstractResource*>>(m_packages.extendedBy[filter.extends], [](AppPackageKitResource* a){ return a; });
+        const auto ext = kTransform<QVector<AbstractResource*>>(m_packages.extendedBy.value(filter.extends), [](AppPackageKitResource* a){ return a; });
         return new ResultsStream(QStringLiteral("PackageKitStream-extends"), ext);
     } else if (filter.search.isEmpty()) {
-        return new ResultsStream(QStringLiteral("PackageKitStream-all"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return !res->isTechnical(); }));
+        return new ResultsStream(QStringLiteral("PackageKitStream-all"), kFilter<QVector<AbstractResource*>>(m_packages.packages, [](AbstractResource* res) { return res->type() != AbstractResource::Technical && !qobject_cast<PackageKitResource*>(res)->extendsItself(); }));
     } else {
-        const QList<AppStream::Component> components = m_appdata->search(filter.search);
-        const QStringList ids = kTransform<QStringList>(components, [](const AppStream::Component& comp) { return comp.id(); });
         auto stream = new ResultsStream(QStringLiteral("PackageKitStream-search"));
-        if (!ids.isEmpty()) {
-            const auto resources = resourcesByPackageNames<QVector<AbstractResource*>>(ids);
-            QTimer::singleShot(0, this, [stream, resources] () {
+        const auto f = [this, stream, filter] () {
+            const QList<AppStream::Component> components = m_appdata->search(filter.search);
+            const QStringList ids = kTransform<QStringList>(components, [](const AppStream::Component& comp) { return comp.id(); });
+            if (!ids.isEmpty()) {
+                const auto resources = kFilter<QVector<AbstractResource*>>(resourcesByPackageNames<QVector<AbstractResource*>>(ids), [](AbstractResource* res){ return !qobject_cast<PackageKitResource*>(res)->extendsItself(); });
                 Q_EMIT stream->resourcesFound(resources);
-            });
-        }
-
-        PackageKit::Transaction * tArch = PackageKit::Daemon::resolve(filter.search, PackageKit::Transaction::FilterArch);
-        connect(tArch, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackageArch);
-        connect(tArch, &PackageKit::Transaction::package, stream, [stream](PackageKit::Transaction::Info /*info*/, const QString &packageId){
-            stream->setProperty("packageId", packageId);
-        });
-        connect(tArch, &PackageKit::Transaction::finished, stream, [stream, ids, this](PackageKit::Transaction::Exit status) {
-            getPackagesFinished();
-            if (status == PackageKit::Transaction::Exit::ExitSuccess) {
-                const auto packageId = stream->property("packageId");
-                if (!packageId.isNull()) {
-                    const auto res = resourcesByPackageNames<QVector<AbstractResource*>>({PackageKit::Daemon::packageName(packageId.toString())});
-                    Q_EMIT stream->resourcesFound(kFilter<QVector<AbstractResource*>>(res, [ids](AbstractResource* res){ return !ids.contains(res->appstreamId()); }));
-                }
             }
-            stream->finish();
-        }, Qt::QueuedConnection);
+
+            PackageKit::Transaction * tArch = PackageKit::Daemon::resolve(filter.search, PackageKit::Transaction::FilterArch);
+            connect(tArch, &PackageKit::Transaction::package, this, &PackageKitBackend::addPackageArch);
+            connect(tArch, &PackageKit::Transaction::package, stream, [stream](PackageKit::Transaction::Info /*info*/, const QString &packageId){
+                stream->setProperty("packageId", packageId);
+            });
+            connect(tArch, &PackageKit::Transaction::finished, stream, [stream, ids, this](PackageKit::Transaction::Exit status) {
+                getPackagesFinished();
+                if (status == PackageKit::Transaction::Exit::ExitSuccess) {
+                    const auto packageId = stream->property("packageId");
+                    if (!packageId.isNull()) {
+                        const auto res = resourcesByPackageNames<QVector<AbstractResource*>>({PackageKit::Daemon::packageName(packageId.toString())});
+                        Q_EMIT stream->resourcesFound(kFilter<QVector<AbstractResource*>>(res, [ids](AbstractResource* res){ return !ids.contains(res->appstreamId()); }));
+                    }
+                }
+                stream->finish();
+            }, Qt::QueuedConnection);
+        };
+        if (!m_appstreamInitialized) {
+            connect(this, &PackageKitBackend::loadedAppStream, stream, f);
+        } else {
+            QTimer::singleShot(0, this, f);
+        }
         return stream;
     }
 }
 
 ResultsStream * PackageKitBackend::findResourceByPackageName(const QUrl& url)
 {
-    AbstractResource* pkg = nullptr;
-    if (url.host().isEmpty())
-        Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
-    else if (url.scheme() == QLatin1String("appstream")) {
+    if (url.isLocalFile()) {
+        QMimeDatabase db;
+        const auto mime = db.mimeTypeForUrl(url);
+        if (    mime.inherits(QStringLiteral("application/vnd.debian.binary-package"))
+            || mime.inherits(QStringLiteral("application/x-rpm"))
+            || mime.inherits(QStringLiteral("application/x-tar"))
+            || mime.inherits(QStringLiteral("application/x-zstd-compressed-tar"))
+            || mime.inherits(QStringLiteral("application/x-xz-compressed-tar"))
+        ) {
+            return new ResultsStream(QStringLiteral("PackageKitStream-localpkg"), QVector<AbstractResource*>{ new LocalFilePKResource(url, this)});
+        }
+    } else if (url.scheme() == QLatin1String("appstream")) {
         static const QMap<QString, QString> deprecatedAppstreamIds = {
             { QStringLiteral("org.kde.krita.desktop"), QStringLiteral("krita.desktop") },
             { QStringLiteral("org.kde.digikam.desktop"), QStringLiteral("digikam.desktop") },
@@ -434,24 +530,40 @@ ResultsStream * PackageKitBackend::findResourceByPackageName(const QUrl& url)
             { QStringLiteral("org.blender.blender.desktop"), QStringLiteral("blender.desktop") },
         };
         
-        const auto host = url.host();
-        if (host.isEmpty())
+        const auto appstreamIds = AppStreamUtils::appstreamIds(url);
+        if (appstreamIds.isEmpty())
             Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
         else {
-            const auto deprecatedHost = deprecatedAppstreamIds.value(host); //try this as fallback
-            for (auto it = m_packages.packages.constBegin(), itEnd = m_packages.packages.constEnd(); it != itEnd; ++it) {
-                if    (it.key().compare(host, Qt::CaseInsensitive) == 0
-                    || it.key().compare(deprecatedHost, Qt::CaseInsensitive) == 0
-                    || (host.endsWith(QLatin1String(".desktop")) && host.compare(it.key()+QLatin1String(".desktop"), Qt::CaseInsensitive) == 0)) {
-                    pkg = it.value();
-                    break;
+            auto stream = new ResultsStream(QStringLiteral("PackageKitStream-appstream-url"));
+            const auto f = [this, appstreamIds, stream] () {
+                AbstractResource* pkg = nullptr;
+
+                const QStringList allAppStreamIds = appstreamIds + deprecatedAppstreamIds.values(appstreamIds.first());
+                for (auto it = m_packages.packages.constBegin(), itEnd = m_packages.packages.constEnd(); it != itEnd; ++it) {
+                    const bool matches = kContains(allAppStreamIds, [&it] (const QString& id) {
+                        return it.key().compare(id, Qt::CaseInsensitive) == 0 ||
+                              (id.endsWith(QLatin1String(".desktop")) && id.compare(it.key()+QLatin1String(".desktop"), Qt::CaseInsensitive) == 0);
+                    });
+                    if (matches) {
+                        pkg = it.value();
+                        break;
+                    }
                 }
+                if (pkg)
+                    Q_EMIT stream->resourcesFound({pkg});
+                stream->finish();
+    //             if (!pkg)
+    //                 qCDebug(LIBDISCOVER_BACKEND_LOG) << "could not find" << host << deprecatedHost;
+            };
+            if (!m_appstreamInitialized) {
+                connect(this, &PackageKitBackend::loadedAppStream, stream, f);
+            } else {
+                QTimer::singleShot(0, this, f);
             }
-            if (!pkg)
-                qDebug() << "could not find" << host << deprecatedHost;
+            return stream;
         }
     }
-    return new ResultsStream(QStringLiteral("PackageKitStream-url"), pkg ? QVector<AbstractResource*>{pkg} : QVector<AbstractResource*>{});
+    return new ResultsStream(QStringLiteral("PackageKitStream-unknown-url"), {});
 }
 
 bool PackageKitBackend::hasSecurityUpdates() const
@@ -461,9 +573,13 @@ bool PackageKitBackend::hasSecurityUpdates() const
 
 int PackageKitBackend::updatesCount() const
 {
+    if (PackageKit::Daemon::global()->offline()->updateTriggered())
+        return 0;
+
     int ret = 0;
     QSet<QString> packages;
-    for(auto res: upgradeablePackages()) {
+    const auto toUpgrade = upgradeablePackages();
+    for(auto res: toUpgrade) {
         const auto packageName = res->packageName();
         if (packages.contains(packageName)) {
             continue;
@@ -477,28 +593,19 @@ int PackageKitBackend::updatesCount() const
 Transaction* PackageKitBackend::installApplication(AbstractResource* app, const AddonList& addons)
 {
     Transaction* t = nullptr;
-    if(!addons.addonsToInstall().isEmpty())
-    {
-        QVector<AbstractResource*> appsToInstall;
-
+    if(!addons.addonsToInstall().isEmpty()) {
+        QVector<AbstractResource*> appsToInstall = resourcesByPackageNames<QVector<AbstractResource*>>(addons.addonsToInstall());
         if(!app->isInstalled())
             appsToInstall << app;
-
-        foreach(const QString& toInstall, addons.addonsToInstall()) {
-            appsToInstall += m_packages.packages.value(toInstall);
-            Q_ASSERT(appsToInstall.last());
-        }
         t = new PKTransaction(appsToInstall, Transaction::ChangeAddonsRole);
-    }
+    } else if (!app->isInstalled())
+        t = installApplication(app);
 
     if (!addons.addonsToRemove().isEmpty()) {
-        QVector<AbstractResource*> appsToRemove = kTransform<QVector<AbstractResource*>>(addons.addonsToRemove(), [this](const QString& toRemove){ return m_packages.packages.value(toRemove); });
+        const auto appsToRemove = resourcesByPackageNames<QVector<AbstractResource*>>(addons.addonsToRemove());
         t = new PKTransaction(appsToRemove, Transaction::RemoveRole);
     }
 
-    if (!app->isInstalled())
-        t = installApplication(app);
-
     return t;
 }
 
@@ -529,7 +636,7 @@ QSet<AbstractResource*> PackageKitBackend::upgradeablePackages() const
         }
         ret.unite(pkgs);
     }
-    return ret;
+    return kFilter<QSet<AbstractResource*>>(ret, [] (AbstractResource* res) { return !static_cast<PackageKitResource*>(res)->extendsItself(); });
 }
 
 void PackageKitBackend::addPackageToUpdate(PackageKit::Transaction::Info info, const QString& packageId, const QString& summary)
@@ -538,6 +645,11 @@ void PackageKitBackend::addPackageToUpdate(PackageKit::Transaction::Info info, c
         return;
     }
 
+    if (info == PackageKit::Transaction::InfoRemoving || info == PackageKit::Transaction::InfoObsoleting) {
+        // Don't try updating packages which need to be removed
+        return;
+    }
+
     if (info == PackageKit::Transaction::InfoSecurity)
         m_hasSecurityUpdates = true;
 
@@ -548,13 +660,20 @@ void PackageKitBackend::addPackageToUpdate(PackageKit::Transaction::Info info, c
 void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
 {
     if (!m_updatesPackageId.isEmpty()) {
+        resolvePackages(kTransform<QStringList>(m_updatesPackageId, [](const QString &pkgid) { return PackageKit::Daemon::packageName(pkgid); }));
         fetchDetails(m_updatesPackageId);
     }
 
     m_updater->setProgressing(false);
 
     includePackagesToAdd();
-    emit updatesCountChanged();
+    if (isFetching()) {
+        auto a = new OneTimeAction([this] {
+            emit updatesCountChanged();
+        }, this);
+        connect(this, &PackageKitBackend::available, a, &OneTimeAction::trigger);
+    } else
+        emit updatesCountChanged();
 }
 
 bool PackageKitBackend::isPackageNameUpgradeable(const PackageKitResource* res) const
@@ -584,7 +703,7 @@ void PackageKitBackend::fetchDetails(const QSet<QString>& pkgid)
 void PackageKitBackend::performDetailsFetch()
 {
     Q_ASSERT(!m_packageNamesToFetchDetails.isEmpty());
-    const auto ids = m_packageNamesToFetchDetails.toList();
+    const auto ids = m_packageNamesToFetchDetails.values();
 
     PackageKit::Transaction* transaction = PackageKit::Daemon::getDetails(ids);
     connect(transaction, &PackageKit::Transaction::details, this, &PackageKitBackend::packageDetails);
@@ -596,7 +715,8 @@ void PackageKitBackend::checkDaemonRunning()
 {
     if (!PackageKit::Daemon::isRunning()) {
         qWarning() << "PackageKit stopped running!";
-    }
+    } else
+        updateProxy();
 }
 
 AbstractBackendUpdater* PackageKitBackend::backendUpdater() const
@@ -614,55 +734,22 @@ AbstractReviewsBackend* PackageKitBackend::reviewsBackend() const
     return m_reviews.data();
 }
 
-AbstractResource * PackageKitBackend::resourceForFile(const QUrl& file)
-{
-    QMimeDatabase db;
-    const auto mime = db.mimeTypeForUrl(file);
-    if (    mime.inherits(QLatin1String("application/vnd.debian.binary-package"))
-         || mime.inherits(QLatin1String("application/x-rpm"))
-         || mime.inherits(QLatin1String("application/x-tar"))
-         || mime.inherits(QLatin1String("application/x-xz-compressed-tar"))
-    ) {
-        return new LocalFilePKResource(file, this);
-    }
-    return nullptr;
-}
-
-static QString readDistroName()
-{
-    const QStringList osreleasenames = (QStringList() << QStringLiteral("/etc/os-release")
-                                                      << QStringLiteral("/usr/lib/os-release"));
-    foreach (QString osrelease, osreleasenames)
-    {
-        QFile file(osrelease);
-        if (file.open(QIODevice::ReadOnly | QIODevice::Text))
-        {
-            QByteArray line;
-            while (!file.atEnd()) {
-                line = file.readLine().trimmed();
-                if (line.startsWith("NAME=")) {
-                    auto output = line.right(line.length()-5);
-                    output = output.replace('\"',"");
-                    return QString::fromLocal8Bit(output);
-                }
-            }
-        }
-    }
-
-    QProcess process;
-    process.setEnvironment({QStringLiteral("LC_ALL=C")});
-    process.start(QStringLiteral("lsb_release"), {QStringLiteral("-sd")});
-    process.waitForFinished();
-    auto output = process.readAll().trimmed();
-    if (output.startsWith('\"') && output.endsWith('\"'))
-        output = output.mid(1, output.length()-2);
-    return QString::fromLocal8Bit(output);
-}
-
 QString PackageKitBackend::displayName() const
 {
-    static const QString distro = readDistroName();
-    return distro;
+    return AppStreamIntegration::global()->osRelease()->prettyName();
+}
+
+int PackageKitBackend::fetchingUpdatesProgress() const
+{
+    if (!m_getUpdatesTransaction)
+        return 0;
+    
+    if (m_getUpdatesTransaction->status() == PackageKit::Transaction::StatusWait || m_getUpdatesTransaction->status() == PackageKit::Transaction::StatusUnknown) {
+        return m_getUpdatesTransaction->property("lastPercentage").toInt();
+    }
+    int percentage = percentageWithStatus(m_getUpdatesTransaction->status(), m_getUpdatesTransaction->percentage());
+    m_getUpdatesTransaction->setProperty("lastPercentage", percentage);
+    return percentage;
 }
 
 #include "PackageKitBackend.moc"
