@@ -27,6 +27,13 @@
 #include <PackageKit/Daemon>
 #include <QJsonArray>
 #include <QDebug>
+#include <utils.h>
+
+#if defined(WITH_MARKDOWN)
+extern "C" {
+#include <mkdio.h>
+}
+#endif
 
 const QStringList PackageKitResource::m_objects({ QStringLiteral("qrc:/qml/DependenciesButton.qml") });
 
@@ -96,10 +103,15 @@ QVariant PackageKitResource::icon() const
     return QStringLiteral("applications-other");
 }
 
-QString PackageKitResource::license()
+QJsonArray PackageKitResource::licenses()
 {
     fetchDetails();
-    return m_details.license().isEmpty() ? i18n("Unknown") : m_details.license();
+
+    if (!m_details.license().isEmpty()) {
+        return { QJsonObject {{ QStringLiteral("name"), m_details.license() }} };
+    }
+
+    return { QJsonObject {{ QStringLiteral("name"), {} }} };
 }
 
 QList<PackageState> PackageKitResource::addonsInformation()
@@ -148,11 +160,14 @@ AbstractResource::State PackageKitResource::state()
 
 void PackageKitResource::addPackageId(PackageKit::Transaction::Info info, const QString &packageId, bool arch)
 {
+    auto oldState = state();
     if (arch)
         m_packages[info].append(packageId);
     else
         m_packages[info].prepend(packageId);
-    emit stateChanged();
+
+    if (oldState != state())
+        emit stateChanged();
 }
 
 QStringList PackageKitResource::categories()
@@ -160,9 +175,9 @@ QStringList PackageKitResource::categories()
     return { QStringLiteral("Unknown") };
 }
 
-bool PackageKitResource::isTechnical() const
+AbstractResource::Type PackageKitResource::type() const
 {
-    return true;
+    return Technical;
 }
 
 void PackageKitResource::fetchDetails()
@@ -175,9 +190,9 @@ void PackageKitResource::fetchDetails()
     backend()->fetchDetails(pkgid);
 }
 
-void PackageKitResource::failedFetchingDetails(PackageKit::Transaction::Error, const QString& msg)
+void PackageKitResource::failedFetchingDetails(PackageKit::Transaction::Error error, const QString& msg)
 {
-    qWarning() << "error fetching details" << msg;
+    qWarning() << "error fetching details" << error << msg;
 }
 
 void PackageKitResource::setDependenciesCount(int deps)
@@ -195,8 +210,11 @@ void PackageKitResource::setDetails(const PackageKit::Details & details)
         return;
 
     if (m_details != details) {
+        const auto oldState = state();
         m_details = details;
-        emit stateChanged();
+
+        if (oldState != state())
+            emit stateChanged();
 
         if (!backend()->isFetching())
             Q_EMIT backend()->resourcesChanged(this, {"size", "homepage", "license"});
@@ -211,7 +229,8 @@ void PackageKitResource::fetchUpdateDetails()
 {
     const auto pkgid = availablePackageId();
     if (pkgid.isEmpty()) {
-        connect(this, &PackageKitResource::stateChanged, this, &PackageKitResource::fetchUpdateDetails);
+        auto a = new OneTimeAction([this] { fetchUpdateDetails(); }, this);
+        connect(this, &PackageKitResource::stateChanged, a, &OneTimeAction::trigger);
         return;
     }
     PackageKit::Transaction* t = PackageKit::Daemon::getUpdateDetail(availablePackageId());
@@ -222,7 +241,7 @@ void PackageKitResource::fetchUpdateDetails()
 static void addIfNotEmpty(const QString& title, const QString& content, QString& where)
 {
     if (!content.isEmpty())
-        where += QStringLiteral("<p><b>") + title + QStringLiteral("</b>&nbsp;") + QString(content).replace(QStringLiteral("\n"), QStringLiteral("<br />")) + QStringLiteral("</p>");
+        where += QLatin1String("<p><b>") + title + QLatin1String("</b>&nbsp;") + QString(content).replace(QLatin1Char('\n'), QLatin1String("<br />")) + QLatin1String("</p>");
 }
 
 QString PackageKitResource::joinPackages(const QStringList& pkgids, const QString &_sep, const QString &shadowPackage)
@@ -247,21 +266,39 @@ static QStringList urlToLinks(const QStringList& urls)
     return ret;
 }
 
-void PackageKitResource::updateDetail(const QString& packageID, const QStringList& updates, const QStringList& obsoletes, const QStringList& vendorUrls,
-                                      const QStringList& /*bugzillaUrls*/, const QStringList& /*cveUrls*/, PackageKit::Transaction::Restart restart, const QString& updateText,
+void PackageKitResource::updateDetail(const QString& packageID, const QStringList& /*updates*/, const QStringList& obsoletes, const QStringList& vendorUrls,
+                                      const QStringList& /*bugzillaUrls*/, const QStringList& /*cveUrls*/, PackageKit::Transaction::Restart restart, const QString &_updateText,
                                       const QString& /*changelog*/, PackageKit::Transaction::UpdateState state, const QDateTime& /*issued*/, const QDateTime& /*updated*/)
 {
+#if defined(WITH_MARKDOWN)
+    const char* xx = _updateText.toUtf8().constData();
+    MMIOT *markdownHandle = mkd_string(xx, _updateText.size(), 0);
+
+    QString updateText;
+    if ( !mkd_compile( markdownHandle, MKD_FENCEDCODE | MKD_GITHUBTAGS | MKD_AUTOLINK ) ) {
+        updateText = _updateText;
+    } else {
+        char *htmlDocument;
+        const int size = mkd_document( markdownHandle, &htmlDocument );
+
+        updateText = QString::fromUtf8( htmlDocument, size );
+    }
+    mkd_cleanup( markdownHandle );
+
+#else
+    const auto& updateText = _updateText;
+#endif
+
     const auto name = PackageKit::Daemon::packageName(packageID);
 
     QString info;
-    addIfNotEmpty(i18n("Current Version:"), joinPackages(updates, {}, name), info);
     addIfNotEmpty(i18n("Obsoletes:"), joinPackages(obsoletes, {}, name), info);
-    addIfNotEmpty(i18n("New Version:"), updateText, info);
+    addIfNotEmpty(i18n("Release Notes:"), updateText, info);
     addIfNotEmpty(i18n("Update State:"), PackageKitMessages::updateStateMessage(state), info);
     addIfNotEmpty(i18n("Restart:"), PackageKitMessages::restartMessage(restart), info);
 
     if (!vendorUrls.isEmpty())
-        addIfNotEmpty(i18n("Vendor:"), urlToLinks(vendorUrls).join(QStringLiteral(", ")), info);
+        addIfNotEmpty(i18n("Vendor:"), urlToLinks(vendorUrls).join(QLatin1String(", ")), info);
 
     emit changelogFetched(changelog() + info);
 }
@@ -286,7 +323,7 @@ QString PackageKitResource::sizeDescription()
 
 QString PackageKitResource::sourceIcon() const
 {
-    return QStringLiteral("package-available");
+    return QStringLiteral("package-x-generic");
 }
 
 void PackageKitResource::fetchDependencies()
@@ -307,3 +344,18 @@ void PackageKitResource::fetchDependencies()
         Q_EMIT dependenciesFound(*packageDependencies);
     });
 }
+
+bool PackageKitResource::extendsItself() const
+{
+    const auto extendsResources = backend()->resourcesByPackageNames<QVector<AbstractResource*>>(extends());
+    if (extendsResources.isEmpty())
+        return false;
+
+    const auto ourPackageNames = allPackageNames();
+    for (auto r : extendsResources) {
+        PackageKitResource* pkres = qobject_cast<PackageKitResource*>(r);
+        if (pkres->allPackageNames() != ourPackageNames)
+            return false;
+    }
+    return true;
+}
