@@ -27,7 +27,7 @@
 #include <KShell>
 #include <QList>
 #include <QProcess>
-#include <QDebug>
+#include "libdiscover_debug.h"
 
 AbstractResource::AbstractResource(AbstractResourcesBackend* parent)
     : QObject(parent)
@@ -38,9 +38,24 @@ AbstractResource::AbstractResource(AbstractResourcesBackend* parent)
 
 AbstractResource::~AbstractResource() = default;
 
-bool AbstractResource::isTechnical() const
+QUrl AbstractResource::homepage()
 {
-    return false;
+    return QUrl();
+}
+
+QUrl AbstractResource::helpURL()
+{
+    return QUrl();
+}
+
+QUrl AbstractResource::bugURL()
+{
+    return QUrl();
+}
+
+QUrl AbstractResource::donationURL()
+{
+    return QUrl();
 }
 
 void AbstractResource::addMetadata(const QString &key, const QJsonValue &value)
@@ -53,12 +68,12 @@ QJsonValue AbstractResource::getMetadata(const QString &key)
     return m_metadata.value(key);
 }
 
-bool AbstractResource::canUpgrade() const
+bool AbstractResource::canUpgrade()
 {
     return state() == Upgradeable;
 }
 
-bool AbstractResource::isInstalled() const
+bool AbstractResource::isInstalled()
 {
     return state() >= Installed;
 }
@@ -123,7 +138,8 @@ void AbstractResource::reportNewState()
     if (backend()->isFetching())
         return;
 
-    emit backend()->resourcesChanged(this, {"state", "status", "canUpgrade", "size", "sizeDescription", "installedVersion", "availableVersion" });
+    static const QVector<QByteArray> ns = {"state", "status", "canUpgrade", "size", "sizeDescription", "installedVersion", "availableVersion" };
+    emit backend()->resourcesChanged(this, ns);
 }
 
 static bool shouldFilter(AbstractResource* res, const QPair<FilterType, QString>& filter)
@@ -212,7 +228,7 @@ QString AbstractResource::categoryDisplay() const
         ret.append(cat->name());
     }
     ret.sort();
-    return ret.join(QStringLiteral(", "));
+    return ret.join(QLatin1String(", "));
 }
 
 QUrl AbstractResource::url() const
@@ -225,3 +241,8 @@ QString AbstractResource::displayOrigin() const
 {
     return i18nc("origin (backend name)", "%1 (%2)", origin(), backend()->displayName());
 }
+
+QString AbstractResource::executeLabel() const
+{
+    return i18n("Launch");
+}
