@@ -31,6 +31,7 @@
 #include "Category/CategoryModel.h"
 #include "utils.h"
 #include "libdiscover_debug.h"
+#include <functional>
 #include <QCoreApplication>
 #include <QThread>
 #include <QAction>
@@ -53,6 +54,28 @@ ResourcesModel::ResourcesModel(QObject* parent, bool load)
     , m_isFetching(false)
     , m_initializingBackends(0)
     , m_currentApplicationBackend(nullptr)
+    , m_allInitializedEmitter(new QTimer(this))
+    , m_updatesCount(0, [this] {
+        {
+            int ret = 0;
+            foreach(AbstractResourcesBackend* backend, m_backends) {
+                ret += backend->updatesCount();
+            }
+            return ret;
+        }
+    }, [this](int count){ Q_EMIT updatesCountChanged(count); })
+    , m_fetchingUpdatesProgress(0, [this] {
+        {
+            if (m_backends.isEmpty())
+                return 0;
+
+            int sum = 0;
+            for(auto backend: m_backends) {
+                sum += backend->fetchingUpdatesProgress();
+            }
+            return sum / m_backends.count();
+        }
+    }, [this](int progress){ Q_EMIT fetchingUpdatesProgressChanged(progress); })
 {
     init(load);
     connect(this, &ResourcesModel::allInitialized, this, &ResourcesModel::slotFetching);
@@ -64,6 +87,13 @@ void ResourcesModel::init(bool load)
     Q_ASSERT(!s_self);
     Q_ASSERT(QCoreApplication::instance()->thread()==QThread::currentThread());
 
+    m_allInitializedEmitter->setSingleShot(true);
+    m_allInitializedEmitter->setInterval(0);
+    connect(m_allInitializedEmitter, &QTimer::timeout, this, [this](){
+        if (m_initializingBackends == 0)
+            emit allInitialized();
+    });
+
     if(load)
         QMetaObject::invokeMethod(this, "registerAllBackends", Qt::QueuedConnection);
 
@@ -74,6 +104,7 @@ void ResourcesModel::init(bool load)
     m_updateAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
     connect(this, &ResourcesModel::fetchingChanged, m_updateAction, [this](bool fetching) {
         m_updateAction->setEnabled(!fetching);
+        m_fetchingUpdatesProgress.reevaluate();
     });
     connect(m_updateAction, &QAction::triggered, this, &ResourcesModel::checkForUpdates);
 
@@ -105,8 +136,7 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend* backend)
 
     m_backends += backend;
     if(!backend->isFetching()) {
-        if (backend->updatesCount() > 0)
-            emit updatesCountChanged();
+        m_updatesCount.reevaluate();
     } else {
         m_initializingBackends++;
     }
@@ -114,15 +144,21 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend* backend)
     connect(backend, &AbstractResourcesBackend::fetchingChanged, this, &ResourcesModel::callerFetchingChanged);
     connect(backend, &AbstractResourcesBackend::allDataChanged, this, &ResourcesModel::updateCaller);
     connect(backend, &AbstractResourcesBackend::resourcesChanged, this, &ResourcesModel::resourceDataChanged);
-    connect(backend, &AbstractResourcesBackend::updatesCountChanged, this, &ResourcesModel::updatesCountChanged);
+    connect(backend, &AbstractResourcesBackend::updatesCountChanged, this, [this] { m_updatesCount.reevaluate(); });
+    connect(backend, &AbstractResourcesBackend::fetchingUpdatesProgressChanged, this, [this] { m_fetchingUpdatesProgress.reevaluate(); });
     connect(backend, &AbstractResourcesBackend::resourceRemoved, this, &ResourcesModel::resourceRemoved);
     connect(backend, &AbstractResourcesBackend::passiveMessage, this, &ResourcesModel::passiveMessage);
     connect(backend->backendUpdater(), &AbstractBackendUpdater::progressingChanged, this, &ResourcesModel::slotFetching);
 
-    if(m_initializingBackends==0)
-        emit allInitialized();
-    else
+    // In case this is in fact the first backend to be added, and also happens to be
+    // pre-filled, we still need for the rest of the backends to be added before trying
+    // to send out the initialized signal. To ensure this happens, schedule it for the
+    // start of the next run of the event loop.
+    if(m_initializingBackends==0) {
+        m_allInitializedEmitter->start();
+    } else {
         slotFetching();
+    }
 }
 
 void ResourcesModel::callerFetchingChanged()
@@ -147,7 +183,7 @@ void ResourcesModel::callerFetchingChanged()
     } else {
         m_initializingBackends--;
         if(m_initializingBackends==0)
-            emit allInitialized();
+            m_allInitializedEmitter->start();
         else
             slotFetching();
     }
@@ -165,17 +201,6 @@ QVector< AbstractResourcesBackend* > ResourcesModel::backends() const
     return m_backends;
 }
 
-int ResourcesModel::updatesCount() const
-{
-    int ret = 0;
-
-    foreach(AbstractResourcesBackend* backend, m_backends) {
-        ret += backend->updatesCount();
-    }
-
-    return ret;
-}
-
 bool ResourcesModel::hasSecurityUpdates() const
 {
     bool ret = false;
@@ -208,7 +233,7 @@ void ResourcesModel::registerAllBackends()
     const auto backends = f.allBackends();
     if(m_initializingBackends==0 && backends.isEmpty()) {
         qCWarning(LIBDISCOVER_LOG) << "Couldn't find any backends";
-        emit allInitialized();
+        m_allInitializedEmitter->start();
     } else {
         foreach(AbstractResourcesBackend* b, backends) {
             addResourcesBackend(b);
@@ -279,6 +304,7 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
     for (auto stream: streams) {
         connect(stream, &ResultsStream::resourcesFound, this, &AggregatedResultsStream::addResults);
         connect(stream, &QObject::destroyed, this, &AggregatedResultsStream::destruction);
+        connect(this, &ResultsStream::fetchMore, stream, &ResultsStream::fetchMore);
         m_streams << stream;
     }
 
@@ -335,17 +361,6 @@ AggregatedResultsStream* ResourcesModel::search(const AbstractResourcesBackend::
     return new AggregatedResultsStream(streams);
 }
 
-AbstractResource* ResourcesModel::resourceForFile(const QUrl& file)
-{
-    AbstractResource* ret = nullptr;
-    foreach(auto backend, m_backends) {
-        ret = backend->resourceForFile(file);
-        if (ret)
-            break;
-    }
-    return ret;
-}
-
 void ResourcesModel::checkForUpdates()
 {
     for(auto backend: qAsConst(m_backends))
@@ -386,8 +401,7 @@ void ResourcesModel::setCurrentApplicationBackend(AbstractResourcesBackend* back
 
 void ResourcesModel::initApplicationsBackend()
 {
-    KConfigGroup settings(KSharedConfig::openConfig(), "ResourcesModel");
-    const QString name = settings.readEntry<QString>("currentApplicationBackend", QStringLiteral("packagekit-backend"));
+    const auto name = applicationSourceName();
 
     const auto backends = applicationBackends();
     auto idx = kIndexOf(backends, [name](AbstractResourcesBackend* b) { return b->name() == name; });
@@ -397,3 +411,9 @@ void ResourcesModel::initApplicationsBackend()
     }
     setCurrentApplicationBackend(backends.value(idx, nullptr), false);
 }
+
+QString ResourcesModel::applicationSourceName() const
+{
+    KConfigGroup settings(KSharedConfig::openConfig(), "ResourcesModel");
+    return settings.readEntry<QString>("currentApplicationBackend", QStringLiteral("packagekit-backend"));
+}
