@@ -45,7 +45,7 @@ public:
             connect(updater, &AbstractBackendUpdater::downloadSpeedChanged, this, &UpdateTransaction::slotDownloadSpeedChanged);
             connect(updater, &AbstractBackendUpdater::progressChanged, this, &UpdateTransaction::slotUpdateProgress);
             connect(updater, &AbstractBackendUpdater::proceedRequest, this, &UpdateTransaction::processProceedRequest);
-            connect(updater, &AbstractBackendUpdater::cancelableChanged, this, [this](bool cancelable){ if (cancelable) setCancellable(true); });
+            connect(updater, &AbstractBackendUpdater::cancelableChanged, this, [this](bool){ setCancellable(kContains(m_allUpdaters, [] (AbstractBackendUpdater* updater) { return updater->isCancelable() && updater->isProgressing(); })); });
             cancelable |= updater->isCancelable();
         }
         setCancellable(cancelable);
@@ -105,7 +105,7 @@ public:
     }
 
     QVariant icon() const override { return QStringLiteral("update-low"); }
-    QString name() const override { return i18n("Update"); }
+    QString name() const override { return i18n("View Updates"); }
 
 Q_SIGNALS:
     void finished();
@@ -137,6 +137,7 @@ void ResourcesUpdatesModel::init()
             connect(updater, &AbstractBackendUpdater::downloadSpeedChanged, this, &ResourcesUpdatesModel::downloadSpeedChanged);
             connect(updater, &AbstractBackendUpdater::resourceProgressed, this, &ResourcesUpdatesModel::resourceProgressed);
             connect(updater, &AbstractBackendUpdater::passiveMessage, this, &ResourcesUpdatesModel::passiveMessage);
+            connect(updater, &AbstractBackendUpdater::needsRebootChanged, this, &ResourcesUpdatesModel::needsRebootChanged);
             connect(updater, &AbstractBackendUpdater::destroyed, this, &ResourcesUpdatesModel::updaterDestroyed);
             m_updaters += updater;
 
@@ -281,4 +282,13 @@ Transaction* ResourcesUpdatesModel::transaction() const
     return m_transaction.data();
 }
 
+bool ResourcesUpdatesModel::needsReboot() const
+{
+    for(auto upd: m_updaters) {
+        if (upd->needsReboot())
+            return true;
+    }
+    return false;
+}
+
 #include "ResourcesUpdatesModel.moc"
